@@ -1034,10 +1034,14 @@ void apu_process(void *buffer, int num_samples)
          accum = -0x8000;
 
       /* signed 16-bit output, unsigned 8-bit */
-      if (16 == apu->sample_bits)
-         *((int16 *) buffer)++ = (int16) accum;
-      else
-         *((uint8 *) buffer)++ = (accum >> 8) ^ 0x80;
+      if (16 == apu->sample_bits) {
+         *(int16 *)(buffer) = (int16) accum;
+         buffer += sizeof(int16);
+      }
+      else {
+         *(uint8 *)(buffer) = (accum >> 8) ^ 0x80;
+         buffer += sizeof(uint8);
+      }
    }
 
    /* resync cycle counter */
